@@ -3,6 +3,10 @@ from ._fixtures import _GenericBackendTest, _GenericMutexTest
 from unittest import TestCase
 from mock import patch, Mock
 import pytest
+import os
+
+REDIS_HOST = '127.0.0.1'
+REDIS_PORT = int(os.getenv('DOGPILE_REDIS_PORT', '6379'))
 
 
 class _TestRedisConn(object):
@@ -25,9 +29,10 @@ class RedisTest(_TestRedisConn, _GenericBackendTest):
     backend = 'dogpile.cache.redis'
     config_args = {
         "arguments": {
-            'host': '127.0.0.1',
-            'port': 6379,
+            'host': REDIS_HOST,
+            'port': REDIS_PORT,
             'db': 0,
+            "foo": "barf"
         }
     }
 
@@ -36,8 +41,8 @@ class RedisDistributedMutexTest(_TestRedisConn, _GenericMutexTest):
     backend = 'dogpile.cache.redis'
     config_args = {
         "arguments": {
-            'host': '127.0.0.1',
-            'port': 6379,
+            'host': REDIS_HOST,
+            'port': REDIS_PORT,
             'db': 0,
             'distributed_lock': True,
         }
@@ -50,16 +55,15 @@ class RedisConnectionTest(TestCase):
 
     @classmethod
     def setup_class(cls):
+        cls.backend_cls = _backend_loader.load(cls.backend)
         try:
-            cls.backend_cls = _backend_loader.load(cls.backend)
             cls.backend_cls({})
         except ImportError:
             pytest.skip("Backend %s not installed" % cls.backend)
 
     def _test_helper(self, mock_obj, expected_args, connection_args=None):
         if connection_args is None:
-            # The redis backend pops items from the dict, so we copy
-            connection_args = expected_args.copy()
+            connection_args = expected_args
 
         self.backend_cls(connection_args)
         mock_obj.assert_called_once_with(**expected_args)
